%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Problem(3-1)-Problem of Non-isothermal Unsteady CSTR Reactor
% Solution by Euler Method

% Set of one algebraiec eq. and five First Order ODEs with initial conditions  
% 1- F=Kv(V-Vmin)
% 2- dV/dt=F0-F
% 3- d(V*CA)/dt=F0*CA0-F*CA-V*K*exp(-E/R*T)CA^n*CB^m
% 4- d(V*CB)/dt=F0*CB0-F*CB-V*K*exp(-E/R*T)CA^n*CB^m
% 5- d(V*T)=[Ro*c*(F0*T0-F*T)-(-dH)*V*K*exp(-E/R*T)CA^n*CB^m-U*Ah(T-Tj)]/(Ro*c)
% 6- dTj/dt=[Fj*Roj*cj*(Tj0-Tj)+U*Ah*(T-Tj)]/(Roj*Vj*cj)

% Initial Conditions t=0 T=Tj=T0 F=F0 V=Vmin CA=CA0 CB=CB0 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear
clc

% Input Constants
const;

% Initial Conditions
CA(1)=CA0;
CB(1)=CB0;
F(1)=F0;
V(1)=Vmin;
Tj(1)=Tj0;
T(1)=Tj0;
T0=Tj0;
t(1)=0;

for i=1:tf/dt-1

% Algebraic equations
    F(i)=Kv*(V(i)-Vmin);

% Differential equations
    dV(i)=dt*(F0-F(i));
    V(i+1)=V(i)+dV(i);

    dVCA(i)=dt*(F0*CA0-F(i)*CA(i)-V(i)*k*exp(-E/(R*T(i)))*CA(i)^n*CB(i)^m);
    CA(i+1)=(V(i)*CA(i)+dVCA(i))/V(i+1);

    dVCB(i)=dt*(F0*CB0-F(i)*CB(i)-V(i)*k*exp(-E/(R*T(i)))*CA(i)^n*CB(i)^m);
    CB(i+1)=(V(i)*CB(i)+dVCB(i))/V(i+1);

    dVT(i)=(dt/(Ro*c))*(Ro*c*(F0*T0-F(i)*T(i))-dH*V(i)*k*exp(-E/(R*T(i)))*CA(i)^n*CB(i)^m-U*Ah*(T(i)-Tj(i)));
    T(i+1)=(V(i)*T(i)+dVT(i))/V(i+1);

    dTj(i)=(dt/(Roj*cj*Vj))*(Fj*Roj*cj*(Tj0-Tj(i))+U*Ah*(T(i)-Tj(i)));
    Tj(i+1)=Tj(i)+dTj(i);

% Time Calculation    
    t(i+1)=t(i)+dt;

% Checking Process State    
    if CA(i+1)<=0
        CA(i+1)=0;
    elseif CB(i+1)<=0
        CB(i+1)=0;
    end
    if abs(CA(i+1)-CA(i))/CA(i)<Tol && abs(T(i+1)-T(i))/T(i)<Tol
        break
    end
end

% Creating End Point for Algebric Equation
F(i+1)=Kv*(V(i)-Vmin);

% Ploting Results
figure

subplot(3,2,1)
plot(t,CA)
title('Concentration of A')
xlabel('t(s)')
ylabel('CA(mol/lit)')

subplot(3,2,3)
plot(t,CB)
title('Concentration of B')
xlabel('t(s)')
ylabel('CB(mol/lit)')

subplot(3,2,2)
plot(t,T-273.15)
title('Temperature')
xlabel('t(s)')
ylabel('T(C)')

subplot(3,2,4)
plot(t,Tj-273.15)
title('Jacket temperature')
xlabel('t(s)')
ylabel('Tj(C)')

subplot(3,2,5)
plot(t,F)
title('Liquid flow')
xlabel('t(s)')
ylabel('F(m3/s)')

subplot(3,2,6)
plot(t,V)
title('Liquid volume')
xlabel('t(s)')
ylabel('V(m3)')

