%%%%%%%%%%%%%%%%%%%%%%%%%Exercise(4-5)%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%    Exercise(4-5)-Unsteady Diffusion and reaction inside the Spherical Catalyst
%    with 2nd order reaction
%    Solution of the PDE b Orthogonal Collocation

%   PDE model Eq.: (1/r^2)*d/dr(r^2*dC/dr)-(k/D)*C^2=(1/D)*dC/dt (Base Form)
%   x=r/R, y=C/C0, tau=t/(R^2/D), phi=k*R^2/D, etha=kc*R/2*D
%   After variable Change: u=x^2
%   The PDE: 4*u*d2y/du2+6*dy/du-phi*y^2=dy/dtau (OC Form)


%   Boundary Conditions:  
%   r=0 dC/dr=0: 
%   r=R:  dC/dr=(K/D)*(C0-C) or: at x=1 dy/dx=(Kc*R/2*D)(1-y)
%   Initial condition:                          
%   t=0 ot tau=0: y=0
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all
clc

global N JJ A B etha phi

% Input Constants
const;
% parameters
phi=K*R^2/D;
etha=0.5*Kc*R/D;

% Solution Parameters
alfa=0;
beta=0.5;
gama0=1;
N=10;

% Calculating Jacobi Roots
J=jacobi_roots(alfa,beta,gama0,N);
JJ=J;
JJ(end+1)=1;

% Creating Differential Matrices Lagrange and its Derivatives 
for i=1:N+1
    A(i,:)=lagrange_diff(JJ(i),JJ,N);
    B(i,:)=lagrange_diff2(JJ(i),JJ,N);
end

% Creating Initial Vectors

t=linspace(0,tf,nt);
tau=t/(R^2/D);
dtau=dt/(R^2/D);

for i=1:N+1
    yi(i,1)=Ci/C0;
end

y(1,:)=yi;

for l=1:nt-1
    k1=dtau*feval(@pdefun,tau(l),y(l,:)');
    k2=dtau*feval(@pdefun,tau(l)+dtau,y(l,:)'+k1);
    y(l+1,:)=y(l,:)+0.5*(k1'+k2');
end

% Creating Final Vector
Cf(2:N+2,:)=y'*C0;
t=linspace(0,tf,nt);
x=linspace(0,1,N+2);
x(2:N+1)=J;
r=x.^0.5;

% Plotting Results
figure
[rr,tt]=meshgrid(r,t);
mesh(rr,tt,Cf')
xlabel('r/R')
ylabel('t(s)')
zlabel('C(kmol/m^3)')

figure
plot(r,Cf,'r')
xlabel('r/R')
ylabel('C(kmol/m^3)')

