clear all
clc

% Experimental Data
Ch2s=[0.011 0.0118 0.021 0.0218 0.0206 0.0211 0.0217 0.0211  0.0105 0.0107 0.022 0.022 0.022]*1e-2;
Ch2=[0.079 0.107 0.136 0.147 0.20 0.236 0.272 0.299 0.204 0.203 0.170 0.190 0.188]*1e-2;
Cdbt=[0.0338 0.0333 0.0334 0.0331 0.0331 0.0329 0.0328 0.0326 0.0082 0.0163 0.0654 0.1 0.134]*1e-2;
rdbt=1e-6*[5.78; 8.22; 10.1; 10.4; 14.6; 17.1; 19.4; 21.4; 5.4; 9.36; 21.6; 31.4; 35.3]*1e-6;

% Reaction Rate Function
f=@(k) k(1)*Cdbt.*Ch2./(1+k(2)*Cdbt+k(3)*Ch2+k(4)*Ch2s).^2;

% Initial Guess
x0=[0.5;0.5;0.5;0.5];

N=length(rdbt);
n=4;
Z=zeros(N,n);
y=rdbt;
i=1;
imax=1000;
x=x0;

% Solver
while i<imax
    for l=1:length(x)
        if x(l)~=0
            dx(l)=0.01*x(l);
        else
            dx(l)=0.01;
        end
    end
    a=x;
    b=x;

    for l=1:length(x)
        a(l)=x(l)-dx(l);
        b(l)=x(l)+dx(l);
        fa=feval(f,a);
        fb=feval(f,b);
        Z(:,l)=(fb-fa)/(b(l)-a(l));
        a(l)=a(l)+dx(l);
        b(l)=b(l)-dx(l);
    end

    M=Z'*Z;
    fi=feval(f,x)';
    D=y-fi;
    dx=inv(M)*Z'*D;
    x=x+dx;


    if max(abs(dx))<1e-15 
        break 
        else
        i=i+1;  
    end
end

fi=feval(f,x)';
D=y-fi;
obf=sum(D)^2;   
% Showing Results
fprintf('No. of iteration = %1.0f\n',i)
fprintf('1st Constant(K1) = %6.6f\n',x(1))
fprintf('2nd Constant(K2) = %6.6f\n',x(2))
fprintf('3nd Constant(K3) = %6.6f\n',x(3))
fprintf('4nd Constant(K4) = %6.6f\n',x(4))
fprintf('OBF*10^27 = %6.6f\n',obf*1e27)
