clear all
clc

% Constants     
k=3.5e-3;
kc=3.5e-4;
D=1e-4;	
N=20;
M=25;
iter=1;
imax=1000;

% Creating Vectors & Step Sizes
r=linspace(0,1,N);
z=linspace(0,1,M);
dr=max(diff(r));
dz=max(diff(z));

% Initial Guess
Cg=ones(N,M);
C=Cg;

% Solver
while iter<imax
    for i=1:N
        for j=1:M
             if i==1
                C(i,:)=(4*C(i+1,:)-C(i+2,:))/3;                        
             elseif i==N 	
                C(i,:)=(4*C(i-1,:)-C(i-2,:)+2*dr*(kc/D))*(D/(3*D+2*dr*kc));
             elseif j==1 
                C(:,j)=1;
             elseif j==M 
                C(:,j)=1;
             else
                C(i,j)=(1/(2/dr^2+2/dz^2+k/D))*((1/dr^2+1/(2*r(i)*dr))*C(i+1,j)+(1/dr^2-1/(2*r(i)*dr))*C(i-1,j)+(1/dz^2)*(C(i,j+1)+C(i,j-1)));
             end
        end
    end
    
    if max(max(abs(Cg-C)))<1e-8
        break
    else
        iter=iter+1;
        Cg=C;
    end
end

% Plotting Result
[R,Z]=meshgrid(r,z);
surf(R,Z,C')
xlabel('r')
ylabel('z')
zlabel('C')
